import os

from types import SimpleNamespace

import dateutil.parser
import json

from bakauditor.tools import iso_to_bytes


def get_borg(ssh=None, pw=None, repo=None):
    ssh_cmd = '' if not ssh else 'ssh {} '.format(ssh)
    with os.popen('{}env BORG_PASSPHRASE={} borg list {} 2>&1'.format(
            ssh_cmd, '' if pw is None else pw, repo)) as p:
        return p.readlines()


def check(**kwargs):
    result = SimpleNamespace(ok=False, time=0, size=None, err=None)
    for r in get_borg(kwargs.get('ssh'), kwargs.get('password'),
                      kwargs['repo']):
        try:
            result.time = max(
                result.time,
                dateutil.parser.parse(r.split(maxsplit=1)[1]).timestamp())
            result.ok = True
        except:
            result.err = 'Command failed'
    return result



import os

from functools import lru_cache
from datetime import datetime

from types import SimpleNamespace


@lru_cache(maxsize=4096)
def get_zfs_snapshots(ssh=None):
    ssh_cmd = '' if not ssh else 'ssh {} '.format(ssh)
    result = []
    with os.popen('{}zfs list -p -t snapshot'.format(ssh_cmd)) as p:
        for s in p.readlines()[1:]:
            try:
                path, size = s.split()[:2]
                result.append((path, size))
            except:
                pass
    return result


def check(**kwargs):
    fs = kwargs['fs']
    t = 0
    size = None
    for snap in get_zfs_snapshots(kwargs.get('ssh')):
        s = snap[0]
        if s.startswith(fs + '@'):
            try:
                t = max(
                    t,
                    datetime.strptime(s.split('@')[1],
                                      kwargs.get('time-fmt')).timestamp())
                try:
                    size = int(snap[1])
                except:
                    pass
            except:
                pass
    return SimpleNamespace(ok=t > 0, time=t, size=size, err=None)



import os

from types import SimpleNamespace

from bakauditor.tools import iso_to_bytes

def check(**kwargs):
    result = SimpleNamespace(ok=False, time=0, size=None, err=None)
    try:
        t = os.path.getmtime(kwargs['path'])
        size = os.path.getsize(kwargs['path'])
    except:
        return result
    result.time = t
    result.size = size
    if 'min-size' in kwargs:
        result.ok = size >= iso_to_bytes(kwargs['min-size'])
        if not result.ok:
            result.err = 'Too small'
    else:
        result.ok = True
    return result
